// RateLoad.cpp : Implementation of CRateLoad
#include "stdafx.h"
#include "loadbal.h"
#include "wchar.h"
#include "RateLoad.h"
#include <atlbase.h>
#include "process.h"

/////////////////////////////////////////////////////////////////////////////
// CRateLoad

STDMETHODIMP CRateLoad::OnStartPage (IUnknown* pUnk)  
{
	if(!pUnk)
		return E_POINTER;

	CComPtr<IScriptingContext> spContext;
	HRESULT hr;

	// Get the IScriptingContext Interface
	hr = pUnk->QueryInterface(IID_IScriptingContext, (void **)&spContext);
	if(FAILED(hr))
		return hr;

	// Get Request Object Pointer
	hr = spContext->get_Request(&m_piRequest);
	if(FAILED(hr))
	{
		spContext.Release();
		return hr;
	}

	// Get Response Object Pointer
	hr = spContext->get_Response(&m_piResponse);
	if(FAILED(hr))
	{
		m_piRequest.Release();
		return hr;
	}
	
	// Get Server Object Pointer
	hr = spContext->get_Server(&m_piServer);
	if(FAILED(hr))
	{
		m_piRequest.Release();
		m_piResponse.Release();
		return hr;
	}
	
	// Get Session Object Pointer
	hr = spContext->get_Session(&m_piSession);
	if(FAILED(hr))
	{
		m_piRequest.Release();
		m_piResponse.Release();
		m_piServer.Release();
		return hr;
	}

	// Get Application Object Pointer
	hr = spContext->get_Application(&m_piApplication);
	if(FAILED(hr))
	{
		m_piRequest.Release();
		m_piResponse.Release();
		m_piServer.Release();
		m_piSession.Release();
		return hr;
	}
	m_bOnStartPageCalled = TRUE;
	return S_OK;
}

STDMETHODIMP CRateLoad::OnEndPage ()  
{
	m_bOnStartPageCalled = FALSE;
	// Release all interfaces
	m_piRequest.Release();
	m_piResponse.Release();
	m_piServer.Release();
	m_piSession.Release();
	m_piApplication.Release();

	return S_OK;
}


STDMETHODIMP CRateLoad::Init(BSTR szObjectName, BSTR szCounterName)
{
	lstrcpy(m_szObjectName, szObjectName);
	lstrcpy(m_szCounterName, szCounterName);	
	return S_OK;
}

STDMETHODIMP CRateLoad::AddServer(BSTR szServerName)
{
	if (m_dwNumServers == MAX_SERVERS-1)
		return E_FAIL;

	if (NULL == szServerName || lstrlen(szServerName)==0)
		return E_FAIL;

	lstrcpy(m_sl[m_dwNumServers].szServerName, szServerName);
	m_sl[m_dwNumServers].dwStatus=SERVER_UNINITIALIZED;
	m_sl[m_dwNumServers].dwValue=0;
	m_dwNumServers++;

	return S_OK;
}

STDMETHODIMP CRateLoad::GetPreferredServer(BSTR* szServerName)
{
	// TODO: Add your implementation code here

	if (m_bSound)
		Beep(2000, 10);

	if (m_dwNumServers <= 0)
		return E_FAIL;

	// Create a temporary CComBSTR
	EnterCriticalSection(&m_cs);
    CComBSTR bstrTemp(m_sl[0].szServerName);
	LeaveCriticalSection(&m_cs);

	if (m_bSound)
		Beep(3000, 10);

    if (!bstrTemp)
        return E_OUTOFMEMORY;

    // Return m_str member of bstrTemp
    *szServerName = bstrTemp.Detach();

	
	if (m_bSound)
		Beep(4000, 10);
	
	return S_OK;
}

STDMETHODIMP CRateLoad::DeInit()
{
	// TODO: Add your implementation code here

	return S_OK;
}



DWORD CRateLoad::GetCounterInfo(LPTSTR szServerName, LPTSTR szObjectName, LPTSTR szCounterName, LPTSTR szObjectID, LPDWORD lpdwCounterID)
{

	DWORD	dwRet=0;
    DWORD	rc;
    HKEY                         hKeyRoot;
    HKEY                         hKeyNames;
    DWORD                        dwType;
    DWORD                        dwSize;
    LPTSTR                       buf = NULL;
    TCHAR                        szSubKey[1024];
    LANGID                       lid;
    LPTSTR                       p;
    LPTSTR                       p2;


    lid = MAKELANGID( LANG_ENGLISH, SUBLANG_NEUTRAL );
    wsprintf( szSubKey, L"%s\\%03x", REGKEY_PERF, lid );
	rc=RegConnectRegistry(szServerName, HKEY_LOCAL_MACHINE, &hKeyRoot);
    
    if (rc != ERROR_SUCCESS) 
		{
		return ERROR_FAILED_KEY_OPEN;
		}

	rc = RegOpenKeyEx( hKeyRoot,
                       szSubKey,
                       0,
                       KEY_READ,
                       &hKeyNames
					   );

	
	//rc = RegOpenKeyEx( HKEY_LOCAL_MACHINE,
    //                   szSubKey,
    //                   0,
    //                   KEY_READ,
    //                   &hKeyNames
    //				     );

    if (rc != ERROR_SUCCESS) 
		{
    	RegCloseKey(hKeyRoot);
		return ERROR_FAILED_KEY_OPEN;
		}


    //
    // get the buffer size for the counter names
    //
    rc = RegQueryValueEx( hKeyNames,
                          REGSUBKEY_COUNTERS,
                          NULL,
                          &dwType,
                          NULL,
                          &dwSize
                        );

    if (rc != ERROR_SUCCESS) 
		{
    	RegCloseKey(hKeyNames);
    	RegCloseKey(hKeyRoot);
        return ERROR_FAILED_KEY_READ;
		}


    //
    // allocate the counter names buffer
    //
    buf = (LPTSTR) malloc( dwSize );
    if (buf == NULL) 
		{
    	RegCloseKey(hKeyNames);
    	RegCloseKey(hKeyRoot);
        return ERROR_FAILED_MEMORY_ALLOC;	//cleanup
		}
    memset( buf, 0, dwSize );


    //
    // read the counter names from the registry
    //
    rc = RegQueryValueEx( hKeyNames,
                          REGSUBKEY_COUNTERS,
                          NULL,
                          &dwType,
                          (LPBYTE)buf,
                          &dwSize
                        );

    if (rc != ERROR_SUCCESS) 
		{
    	RegCloseKey(hKeyNames);
    	RegCloseKey(hKeyRoot);
        return ERROR_FAILED_KEY_READ;
		}


    //
    // now loop thru the counter names looking for the following counters:
    //
    //      1.  szObjectName        object name
    //      2.  szCountername       counter name
    //
    // the buffer contains multiple null terminated strings and then
    // finally null terminated at the end.  the strings are in pairs of
    // counter number and counter name.
    //

	bool bObjectFound=FALSE;
	bool bCounterFound=FALSE;

    p = buf;
    while (*p) {
        if (p > buf) 
			{
            for( p2=p-2; iswdigit(*p2); p2--) ;
            }
        if (lstrcmpi(p, szObjectName) == 0) 
			{
            //
            // look backwards for the counter number
            //
            for( p2=p-2; iswdigit(*p2); p2--) ;
            lstrcpy( szObjectID, p2+1 );
			bObjectFound=TRUE;

			//if (!bObjectFound)
			//	continue;	//don't look for counter until object is found
							//counter names are only unique per object
			}
        else
			if (bObjectFound && lstrcmpi(p, szCounterName) == 0) 
				{
				//
				// look backwards for the counter number
				//
				for( p2=p-2; iswdigit(*p2); p2--) ;
				*lpdwCounterID = _wtol( p2+1 );
				bCounterFound=TRUE;
				break;	//we're done, don't risk finding a duplicate counter name
				}
        //
        // next string
        //
        p += (lstrlen(p) + 1);
    }

    //
    // free the counter names buffer
    //
    free( buf );

	if (!bObjectFound || !bCounterFound)
		dwRet= ERROR_INVALID_COUNTER;

   	RegCloseKey(hKeyNames);
   	RegCloseKey(hKeyRoot);

	return dwRet;

}

DWORD CRateLoad::GetPerfData(LPTSTR szServerName, LPTSTR szObjectID, DWORD dwCounterID)
{
	DWORD						 dwObjectID ;
    DWORD                        rc;
    DWORD                        dwValue=0;
	HKEY						 hKeyRoot;
    DWORD                        dwType;
    DWORD                        dwSize;
	BYTE						 bBuf[100024];
    PPERF_DATA_BLOCK             pPerf;
    PPERF_OBJECT_TYPE            pObj;
    PPERF_COUNTER_BLOCK          pCounter;
    PPERF_COUNTER_DEFINITION     pCounterDef;


	//Connect to the remote machine

	rc=RegConnectRegistry(szServerName, HKEY_PERFORMANCE_DATA, &hKeyRoot);

	if (rc != ERROR_SUCCESS)
		{
		return ERROR_FAILED_KEY_OPEN;	
		}

    dwSize=100000;
	memset( bBuf, 0, dwSize );
    dwType=REG_DWORD;
	rc=RegQueryValueEx(	hKeyRoot, 
						szObjectID,
						NULL,
						&dwType,
                        bBuf,
                        &dwSize
						);

	if (rc != ERROR_SUCCESS)
		{
    	RegCloseKey(hKeyRoot);
        if ( rc == ERROR_MORE_DATA)
            {
            //TCHAR szBuf[128];
            //wsprintf(szBuf, L"dwType:%ld, dwSize:%ld, REG_DWORD:%ld", dwType, dwSize, REG_DWORD);
            //MessageBox(NULL,  szBuf,L"GetPerfData", MB_OK | MB_SERVICE_NOTIFICATION);
            return ERROR_SHORT_BUFFER;
            }
        else
		    return ERROR_FAILED_KEY_READ;	
		}

    //TCHAR szBuf[128];
    //wsprintf(szBuf, L"dwType:%ld, dwSize:%ld, REG_BINARY:%ld", dwType, dwSize, REG_BINARY);
    //MessageBox(NULL,  szBuf,L"GetPerfData", MB_OK | MB_SERVICE_NOTIFICATION);

        // Read the counter data
    pPerf= (PPERF_DATA_BLOCK)bBuf;

    if (NULL == pPerf ||
        (pPerf)->Signature[0] != (WCHAR)'P' ||
        (pPerf)->Signature[1] != (WCHAR)'E' ||
        (pPerf)->Signature[2] != (WCHAR)'R' ||
        (pPerf)->Signature[3] != (WCHAR)'F' )
        {
    	RegCloseKey(hKeyRoot);
        return ERROR_INVALID_PERF_BLOCK;  //Bad pointer
        }

    //wsprintf(szBuf, L"NumObjectTypes:%ld, HeaderLength:%ld", pPerf->NumObjectTypes, pPerf->HeaderLength);
    //MessageBox(NULL,  szBuf,L"GetPerfData", MB_OK | MB_SERVICE_NOTIFICATION);

    pObj = (PPERF_OBJECT_TYPE)((LPBYTE)pPerf + pPerf->HeaderLength);
    //wsprintf(szBuf, L"NumInstances:%ld, HeaderLength:%ld, ObjectHelpTitleIndex:%ld, ObjectNameTitleIndex:%ld", pObj->NumInstances, pObj->HeaderLength, pObj->ObjectHelpTitleIndex, pObj->ObjectNameTitleIndex);
    //MessageBox(NULL,  szBuf,L"GetPerfData", MB_OK | MB_SERVICE_NOTIFICATION);

	// Walk the returned list of object types for the one we want
	DWORD x;
	dwObjectID = _wtol(szObjectID);	// convert string to number for comparisons
	for (x=0; x < pPerf->NumObjectTypes; x++)
		{
		if (pObj->ObjectNameTitleIndex == dwObjectID)
			break;
		pObj=(PPERF_OBJECT_TYPE)((LPBYTE)pObj + pObj->TotalByteLength);	//next object
		}

    
	bool bCounterFound=FALSE;
	pCounterDef = (PPERF_COUNTER_DEFINITION)((LPBYTE)pObj + pObj->HeaderLength);
	for (x=0; x < pObj->NumCounters; x++)
		{
		//wsprintf(szBuf, L"CounterSize:%ld, CounterOffset:%ld, CounterHelpTitleIndex:%ld, CounterNameTitleIndex:%ld", pCounterDef->CounterSize, pCounterDef->CounterOffset, pCounterDef->CounterHelpTitleIndex, pCounterDef->CounterNameTitleIndex);
		//MessageBox(NULL,  szBuf,L"GetPerfData", MB_OK | MB_SERVICE_NOTIFICATION);
		if (pCounterDef->CounterNameTitleIndex == dwCounterID) //1906)//1998)
			{
			bCounterFound=TRUE;
			break;
			}
		pCounterDef++;
		}
    
	
	if (bCounterFound)
		{
		if (pCounterDef->CounterSize = 4)	// We only handle DWORD counters
			{
			pCounter = (PERF_COUNTER_BLOCK *)((PCHAR)pObj + pObj->DefinitionLength);
			dwValue = * ((PCHAR)pCounter + pCounterDef->CounterOffset);
			}
		else
			{
			dwValue = ERROR_UNSUPPORTED_COUNTER_TYPE;	//Counter not 4 byte number
			}
		}
	else
		{
		dwValue = ERROR_COUNTER_NOT_FOUND;	//Counter not found
		}

	//RegFlushKey(hKeyRoot);
	RegCloseKey(hKeyRoot);
	
	return dwValue;

}

DWORD CRateLoad::SortServers(void)
{
 	if (m_bAscending)
		qsort(m_sl, m_dwNumServers, sizeof(SERVERSTATS), CompareAscending);
	else
		qsort(m_sl, m_dwNumServers, sizeof(SERVERSTATS), CompareDescending);

	return 0;
}


int CompareDescending(const void * lpSS1, const void * lpSS2)
{
	// Regardless of  sort direction, force all servers with non-OK status
	// down the ranking list. Primary sort criterion is dwValue, the PerMon reading.

	int x;
	if (((LPSERVERSTATS)lpSS1)->dwStatus == SERVER_OK)
		{
		if (((LPSERVERSTATS)lpSS2)->dwStatus == SERVER_OK)
			{
			x = ((LPSERVERSTATS)lpSS2)->dwValue - ((LPSERVERSTATS)lpSS1)->dwValue;	
			//x = x==0 ? 1 : x;
			}
		else
			{
			x = (-1);
			}
		}
	else
		{
		if (((LPSERVERSTATS)lpSS2)->dwStatus == SERVER_OK)
			{
			x = 1;
			}
		else
			{
			// Both unreachable, rate by best (lowest) status
			x = ((LPSERVERSTATS)lpSS1)->dwStatus - ((LPSERVERSTATS)lpSS2)->dwStatus;
			}
		}
	
	return x;  
}

int CompareAscending(const void * lpSS1, const void * lpSS2)
{
	int x;
	if (((LPSERVERSTATS)lpSS1)->dwStatus == SERVER_OK)
		{
		if (((LPSERVERSTATS)lpSS2)->dwStatus == SERVER_OK)
			{
			x = ((LPSERVERSTATS)lpSS1)->dwValue - ((LPSERVERSTATS)lpSS2)->dwValue;
			//x = x==0 ? 1 : x;
			}
		else
			{
			x = (-1);
			}
		}
	else
		{
		if (((LPSERVERSTATS)lpSS2)->dwStatus == SERVER_OK)
			{
			x = 1;
			}
		else
			{
			// Both unreachable, rate by best (lowest) status
			x = ((LPSERVERSTATS)lpSS1)->dwStatus - ((LPSERVERSTATS)lpSS2)->dwStatus;
			}
		}
	
	return  x;
}


STDMETHODIMP CRateLoad::ReadCounter(BSTR szServerName, BSTR szObjectName, BSTR szCounterName, LONG *lpdwValue)
{
	// TODO: Add your implementation code here
	TCHAR szObjectID[80];
	DWORD dwCounterID;

	
	*lpdwValue=GetCounterInfo(szServerName, szObjectName, szCounterName, szObjectID, &dwCounterID);
	if (*lpdwValue < 0 )
		return S_OK;		// Error is the negative return counter value 

	
	*lpdwValue=GetPerfData(szServerName,  szObjectID, dwCounterID);  //L"1972");

	return S_OK;
}

void PollingLoop(void *ptr)
	{ 
	HRESULT   hr;
	DWORD	  x=0;
	LONG     dwValue=0;
	CRateLoad *rl;

	//SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_LOWEST);

	rl=(CRateLoad*)ptr; // 'this' pointer to class object
	
	while (rl->m_bKeepPolling)	//(x=0; x<m_dwNumServers; x++)
		{
		hr=rl->ReadCounter(rl->m_sl[x].szServerName, rl->m_szObjectName, rl->m_szCounterName, &dwValue);
		if (dwValue >= 0)
			rl->m_sl[x].dwStatus=SERVER_OK;				// Counter read OK
		else
			rl->m_sl[x].dwStatus=SERVER_UNREACHABLE;	// Failed to read counter

		rl->m_sl[x].dwValue=dwValue;					// Store perf counter
		x++;
		
		if (rl->m_bSound)
			Beep(1000, 100);
		
		Sleep(10);		// Avoid race condition
				
		if (x >= rl->m_dwNumServers)
			{
			EnterCriticalSection(&(rl->m_cs));
			rl->SortServers();							//Re-rank the server list
			LeaveCriticalSection(&(rl->m_cs));
			x=0;
			}
		}

	if (rl->m_bSound)
		{
		Beep(4000, 10);		// creates cascade sound
		Beep(3000, 10);
		Beep(2000, 10);
		Beep(1000, 10);
		Beep(800, 10);
		}

	return;
}

STDMETHODIMP CRateLoad::Start()
{
	m_bKeepPolling=true;		// void( __cdecl *start_address )( void * )
	_beginthread(PollingLoop, 0, this);
	return S_OK;
}

STDMETHODIMP CRateLoad::Stop()
{
	m_bKeepPolling=false;
	return S_OK;
}


STDMETHODIMP CRateLoad::get_SortAscending(BOOL * pVal)
{
	// TODO: Add your implementation code here
	*pVal=m_bAscending;
	return S_OK;
}

STDMETHODIMP CRateLoad::put_SortAscending(BOOL newVal)
{
	m_bAscending=newVal;
	return S_OK;
}

STDMETHODIMP CRateLoad::get_Sound(BOOL * pVal)
{
	*pVal=m_bSound;
	return S_OK;
}

STDMETHODIMP CRateLoad::put_Sound(BOOL newVal)
{
	m_bSound=newVal;
	return S_OK;
}
