// RateLoad.h : Declaration of the CRateLoad

//E:\Program Files\DevStudio\MyProjects\loadbal\ReleaseUMinDependency\Project1.exe

#ifndef __RATELOAD_H_
#define __RATELOAD_H_

#include "resource.h"       // main symbols
#include <asptlb.h>         // Active Server Pages Definitions

//-----------

#define MAX_SERVERS			100
#define INITIAL_SIZE        51200
#define EXTEND_SIZE         25600
#define REGKEY_PERF         L"software\\microsoft\\windows nt\\currentversion\\perflib"
#define REGSUBKEY_COUNTERS  L"Counters"

//Server status values
#define SERVER_OK				0
#define SERVER_UNINITIALIZED	1
#define SERVER_UNREACHABLE		2
#define SERVER_PROCESS_DOWN		3
#define SERVER_DELETED			4

// Error codes
#define ERROR_FAILED_KEY_OPEN			-1
#define ERROR_FAILED_KEY_READ			-2
#define ERROR_FAILED_MEMORY_ALLOC		-3
#define ERROR_INVALID_COUNTER			-4
#define ERROR_SHORT_BUFFER				-5
#define ERROR_INVALID_PERF_BLOCK		-6
#define ERROR_UNSUPPORTED_COUNTER_TYPE	-7
#define ERROR_COUNTER_NOT_FOUND			-8
#define MAX_ERROR						-8

typedef struct
	{
	TCHAR	szServerName[80];
	signed long	dwValue;
	signed long	dwStatus;
	}SERVERSTATS;

typedef SERVERSTATS *LPSERVERSTATS;

//sort functions
extern "C"
{
int   CompareDescending(const void * lpSS1, const void * lpSS2);
int   CompareAscending(const void * lpSS1, const void * lpSS2);
void  PollingLoop(void *ptr);
}
//-----------


/////////////////////////////////////////////////////////////////////////////
// CRateLoad
class ATL_NO_VTABLE CRateLoad : 
	public CComObjectRootEx<CComMultiThreadModel>,
	public CComCoClass<CRateLoad, &CLSID_RateLoad>,
	public IDispatchImpl<IRateLoad, &IID_IRateLoad, &LIBID_LOADBALLib>
{
public:
	CRateLoad()
		{	 
		m_pUnkMarshaler = NULL;

		m_bOnStartPageCalled = FALSE;
		m_bAscending = TRUE;
		m_bSound = FALSE;
		m_bKeepPolling = FALSE;
		m_dwNumServers = 0;
		m_szObjectName[0]=(TCHAR)0;
		m_szCounterName[0]=(TCHAR)0;
		InitializeCriticalSection(&m_cs);
		}
	~CRateLoad()
		{
		m_bKeepPolling = FALSE;
		if (m_bSound)
			Beep(500, 100);
		//DeleteCriticalSection(&m_cs); 
		}

public:

DECLARE_REGISTRY_RESOURCEID(IDR_RATELOAD)
DECLARE_GET_CONTROLLING_UNKNOWN()

BEGIN_COM_MAP(CRateLoad)
	COM_INTERFACE_ENTRY(IRateLoad)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY_AGGREGATE(IID_IMarshal, m_pUnkMarshaler.p)
END_COM_MAP()

	HRESULT FinalConstruct()
	{
		return CoCreateFreeThreadedMarshaler(
			GetControllingUnknown(), &m_pUnkMarshaler.p);
	}

	void FinalRelease()
	{
		m_pUnkMarshaler.Release();
	}

	CComPtr<IUnknown> m_pUnkMarshaler;

// IRateLoad
public:
	STDMETHOD(get_Sound)(/*[out, retval]*/ BOOL *pVal);
	STDMETHOD(put_Sound)(/*[in]*/ BOOL newVal);
	STDMETHOD(get_SortAscending)(/*[out, retval]*/ BOOL *pVal);
	STDMETHOD(put_SortAscending)(/*[in]*/ BOOL newVal);
	STDMETHOD(Stop)(void);
	STDMETHOD(Start)(void);
	STDMETHOD(ReadCounter)(BSTR szServerName, BSTR szObjectName, BSTR szCounterName, LONG /*[out, retval]*/ *lpdwValue);
	STDMETHOD(DeInit)(void);
	STDMETHOD(GetPreferredServer)(BSTR* szServerName);
	STDMETHOD(AddServer)(BSTR szServerName);
	STDMETHOD(Init)(BSTR szObjectName, BSTR szCounterName);
	//Active Server Pages Methods
	STDMETHOD(OnStartPage)(IUnknown* IUnk);
	STDMETHOD(OnEndPage)();

	DWORD m_dwNumServers;				// current number of polled servers
	bool m_bKeepPolling;
	TCHAR m_szCounterName[80];
	TCHAR m_szObjectName[80];
	SERVERSTATS m_sl[MAX_SERVERS];		// server list (make memeory dynamic!)
	DWORD SortServers(void);
	CRITICAL_SECTION m_cs;
	BOOL m_bSound;						// if true, use debugging sounds

private:
	CComPtr<IRequest> m_piRequest;					//Request Object
	CComPtr<IResponse> m_piResponse;				//Response Object
	CComPtr<ISessionObject> m_piSession;			//Session Object
	CComPtr<IServer> m_piServer;					//Server Object
	CComPtr<IApplicationObject> m_piApplication;	//Application Object
	BOOL m_bOnStartPageCalled;						//OnStartPage successful?
protected:
	BOOL m_bAscending;					// sort order for ranking perf data
	DWORD GetCounterInfo(LPTSTR szServerName, LPTSTR szObjectName, LPTSTR szCounterName, LPTSTR szObjectID, LPDWORD lpdwCounterID);
	DWORD GetPerfData(LPTSTR szServerName, LPTSTR szObjectID, DWORD dwCounterID);
};

#endif //__RATELOAD_H_
